import { IModelType, IArrayType, IStateTreeNode, ModelProperties, ModelInstanceType } from 'mobx-state-tree';
/** Interface with all IModelType static and dynamic members,
  * callable to construct a specialized instance. */
export interface ModelInterface<PROPS extends ModelProperties, OTHERS, CustomC, CustomS> {
    new (): IStateTreeNode & IModelType<PROPS, OTHERS, CustomC, CustomS> & ModelInstanceType<PROPS, OTHERS>;
}
export declare let typeTag: string | undefined;
export declare function setTypeTag(tag?: string): void;
/** Force TypeScript to accept an MST model as a superclass.
  * @param model Model (MST tree node) */
export declare function shim<PROPS extends ModelProperties, OTHERS, CustomC, CustomS>(Model: IModelType<PROPS, OTHERS, CustomC, CustomS>, Parent?: any): ModelInterface<PROPS, OTHERS, CustomC, CustomS>;
/** Decorator for actions. By default the mst function treats methods as views. */
export declare function action(target: {
    [key: string]: any;
}, key: string): void;
export interface ClassyOptions {
    name?: string;
    sealed?: boolean;
}
/** Add methods from an ES6 class into an MST model.
  * @param code Class with methods to add as views (the default)
  *   and actions (if decorated).
  * @param data MST model with properties.
  * @param modelName Model name for debugging and polymorphic type tags in snapshots. */
export declare function mst<PROPS extends ModelProperties, OTHERS, CustomC, CustomS, TYPE>(Code: new () => TYPE, Data: IModelType<PROPS, OTHERS, CustomC, CustomS>, modelName?: string | ClassyOptions, options?: ClassyOptions): IModelType<PROPS, TYPE, CustomC, CustomS>;
export declare function polymorphic<PROPS extends ModelProperties, OTHERS, CustomC, CustomS, TYPE>(Code: new () => TYPE, Model: IModelType<PROPS, OTHERS, CustomC, CustomS>, modelName?: string): IModelType<PROPS, TYPE, CustomC, CustomS>;
export declare type RecursiveType<PROPS> = PROPS & {
    children?: RecursiveType<PROPS>[] | null;
};
export declare function mstWithChildren<PROPS extends ModelProperties, OTHERS, CustomC, CustomS, TYPE>(Code: new () => TYPE, Data: IModelType<PROPS, OTHERS, CustomC, CustomS>, name?: string): {
    Model: IModelType<PROPS & {
        children: import("mobx-state-tree").IMaybe<IArrayType<IModelType<RecursiveType<PROPS>, OTHERS, RecursiveType<import("mobx-state-tree")._CustomOrOther<CustomC, import("mobx-state-tree").ModelCreationType<import("mobx-state-tree/dist/internal").ExtractCFromProps<PROPS>>>>, RecursiveType<import("mobx-state-tree")._CustomOrOther<CustomS, import("mobx-state-tree").ModelSnapshotType<PROPS>>>>>>;
    }, TYPE, CustomC, CustomS>;
    Children: IArrayType<any>;
};
